@@ -1,21 +1,10 @@
-/*  This file is part of the KDE libraries
- *  Copyright (C) 1999, 2007 David Faure <faure@kde.org>
- *                1999 Waldo Bastian <bastian@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- **/
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1999, 2007 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 1999 Waldo Bastian <bastian@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kbuildservicefactory_p.h"
 #include "kbuildservicegroupfactory_p.h"
@@ -24,17 +13,17 @@
 #include "ksycoca.h"
 #include "ksycocadict_p.h"
 #include "ksycocaresourcelist_p.h"
-#include "kdesktopfile.h"
+#include <KDesktopFile>
 #include "kservicetype.h"
 #include "sycocadebug.h"
 
 #include <QDebug>
 #include <QDir>
-#include <qmimedatabase.h>
+#include <QMimeDatabase>
 
 #include <assert.h>
 #include <kmimetypefactory_p.h>
-#include <qstandardpaths.h>
+#include <QStandardPaths>
 
 KBuildServiceFactory::KBuildServiceFactory(KServiceTypeFactory *serviceTypeFactory,
         KBuildMimeTypeFactory *mimeTypeFactory,
@@ -48,10 +37,9 @@ KBuildServiceFactory::KBuildServiceFactory(KServiceTypeFactory *serviceTypeFacto
     m_mimeTypeFactory(mimeTypeFactory),
     m_serviceGroupFactory(serviceGroupFactory)
 {
-    m_resourceList = new KSycocaResourceList();
     // We directly care about services desktop files.
     // All the application desktop files are parsed on demand from the vfolder menu code.
-    m_resourceList->add("services", QStringLiteral("kservices5"), QStringLiteral("*.desktop"));
+    m_resourceList.emplace_back("services", QStringLiteral("kservices5"), QStringLiteral("*.desktop"));
 
     m_nameDict = new KSycocaDict();
     m_relNameDict = new KSycocaDict();
@@ -60,7 +48,6 @@ KBuildServiceFactory::KBuildServiceFactory(KServiceTypeFactory *serviceTypeFacto
 
 KBuildServiceFactory::~KBuildServiceFactory()
 {
-    delete m_resourceList;
 }
 
 KService::Ptr KBuildServiceFactory::findServiceByDesktopName(const QString &name)
@@ -154,45 +141,41 @@ void KBuildServiceFactory::save(QDataStream &str)
 
 void KBuildServiceFactory::collectInheritedServices()
 {
-    // For each mimetype, go up the parent-mimetype chains and collect offers.
+    // For each MIME type, go up the parent MIME type chains and collect offers.
     // For "removed associations" to work, we can't just grab everything from all parents.
     // We need to process parents before children, hence the recursive call in
     // collectInheritedServices(mime) and the QSet to process a given parent only once.
     QSet<QString> visitedMimes;
-    Q_FOREACH (const QString &mimeType, m_mimeTypeFactory->allMimeTypes()) {
-        collectInheritedServices(mimeType, visitedMimes);
+    const auto lst = m_mimeTypeFactory->allMimeTypes();
+    for (const QString &mimeType : lst) {
+        collectInheritedServices(mimeType, visitedMimes, 0);
     }
 }
 
-void KBuildServiceFactory::collectInheritedServices(const QString &mimeTypeName, QSet<QString> &visitedMimes)
+void KBuildServiceFactory::collectInheritedServices(const QString &mimeTypeName, QSet<QString> &visitedMimes, int mimeTypeInheritanceLevel)
 {
     if (visitedMimes.contains(mimeTypeName)) {
         return;
     }
     visitedMimes.insert(mimeTypeName);
 
-    // With multiple inheritance, the "mimeTypeInheritanceLevel" isn't exactly
-    // correct (it should only be increased when going up a level, not when iterating
-    // through the multiple parents at a given level). I don't think we care, though.
-    int mimeTypeInheritanceLevel = 0;
-
     QMimeDatabase db;
-    QMimeType qmime = db.mimeTypeForName(mimeTypeName);
-    Q_FOREACH (QString parentMimeType, qmime.parentMimeTypes()) {
+    const QMimeType qmime = db.mimeTypeForName(mimeTypeName);
+    const auto lst = qmime.parentMimeTypes();
+    for (QString parentMimeType : lst) {
 
         // Workaround issue in shared-mime-info and/or Qt, which sometimes return an alias as parent
         parentMimeType = db.mimeTypeForName(parentMimeType).name();
 
-        collectInheritedServices(parentMimeType, visitedMimes);
+        collectInheritedServices(parentMimeType, visitedMimes, mimeTypeInheritanceLevel + 1);
 
-        ++mimeTypeInheritanceLevel;
         const QList<KServiceOffer> &offers = m_offerHash.offersFor(parentMimeType);
         QList<KServiceOffer>::const_iterator itserv = offers.begin();
         const QList<KServiceOffer>::const_iterator endserv = offers.end();
         for (; itserv != endserv; ++itserv) {
             if (!m_offerHash.hasRemovedOffer(mimeTypeName, (*itserv).service())) {
                 KServiceOffer offer(*itserv);
-                offer.setMimeTypeInheritanceLevel(mimeTypeInheritanceLevel);
+                offer.setMimeTypeInheritanceLevel(mimeTypeInheritanceLevel + 1);
                 //qCDebug(SYCOCA) << "INHERITANCE: Adding service" << (*itserv).service()->entryPath() << "to" << mimeTypeName << "mimeTypeInheritanceLevel=" << mimeTypeInheritanceLevel;
                 m_offerHash.addServiceOffer(mimeTypeName, offer);
             }
@@ -265,14 +248,20 @@ void KBuildServiceFactory::populateServiceTypes()
     for (; itserv != endserv; ++itserv) {
 
         KService::Ptr service(static_cast<KService*>((*itserv).data()));
+        const bool hidden = !service->showInCurrentDesktop();
         QVector<KService::ServiceTypeAndPreference> serviceTypeList = service->_k_accessServiceTypes();
         //bool hasAllAll = false;
         //bool hasAllFiles = false;
 
-        // Add this service to all its servicetypes (and their parents) and to all its mimetypes
+        // Add this service to all its servicetypes (and their parents) and to all its MIME types
         for (int i = 0; i < serviceTypeList.count() /*don't cache it, it can change during iteration!*/; ++i) {
             const QString stName = serviceTypeList[i].serviceType;
-            // It could be a servicetype or a mimetype.
+
+            if (hidden && stName != QLatin1String("Application")) {
+                continue;
+            }
+
+            // It could be a servicetype or a MIME type.
             KServiceType::Ptr serviceType = m_serviceTypeFactory->findServiceTypeByName(stName);
             if (serviceType) {
                 const int preference = serviceTypeList[i].preference;
@@ -282,9 +271,17 @@ void KBuildServiceFactory::populateServiceTypes()
                 }
 
                 //qCDebug(SYCOCA) << "Adding service" << service->entryPath() << "to" << serviceType->name() << "pref=" << preference;
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 69)
                 m_offerHash.addServiceOffer(stName, KServiceOffer(service, preference, 0, service->allowAsDefault()));
+#else
+                m_offerHash.addServiceOffer(stName, KServiceOffer(service, preference, 0));
+#endif
             } else {
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 69)
                 KServiceOffer offer(service, serviceTypeList[i].preference, 0, service->allowAsDefault());
+#else
+                KServiceOffer offer(service, serviceTypeList[i].preference, 0);
+#endif
                 QMimeType mime = db.mimeTypeForName(stName);
                 if (!mime.isValid()) {
                     if (stName.startsWith(QLatin1String("x-scheme-handler/"))) {
@@ -292,26 +289,27 @@ void KBuildServiceFactory::populateServiceTypes()
                         m_mimeTypeFactory->createFakeMimeType(stName);
                         m_offerHash.addServiceOffer(stName, offer);
                     } else {
-                        //qCDebug(SYCOCA) << service->entryPath() << "specifies undefined mimetype/servicetype" << stName;
+                        //qCDebug(SYCOCA) << service->entryPath() << "specifies undefined MIME type/servicetype" << stName;
                         // technically we could call addServiceOffer here, 'mime' isn't used. But it
-                        // would be useless, since we have no mimetype entry where to write the offers offset.
+                        // would be useless, since we have no MIME type entry where to write the offers offset.
                         continue;
                     }
                 } else {
                     bool shouldAdd = true;
-                    foreach (const QString &otherType, service->serviceTypes()) {
+                    const auto lst = service->serviceTypes();
+                    for (const QString &otherType : lst) {
                         // Skip derived types if the base class is listed (#321706)
                         if (stName != otherType && mime.inherits(otherType)) {
-                            // But don't skip aliases (they got resolved into mime->name() already, but don't let two aliases cancel out)
+                            // But don't skip aliases (they got resolved into mime.name() already, but don't let two aliases cancel out)
                             if (db.mimeTypeForName(otherType).name() != mime.name()) {
-                                //qCDebug(SYCOCA) << "Skipping" << mime->name() << "because of" << otherType << "(canonical" << KMimeTypeRepository::self()->canonicalName(otherType) << ") while parsing" << service->entryPath();
+                                //qCDebug(SYCOCA) << "Skipping" << mime.name() << "because of" << otherType << "(canonical" << db.mimeTypeForName(otherType) << ") while parsing" << service->entryPath();
                                 shouldAdd = false;
                             }
                         }
                     }
                     if (shouldAdd) {
-                        //qCDebug(SYCOCA) << "Adding service" << service->entryPath() << "to" << mime->name();
-                        m_offerHash.addServiceOffer(mime.name(), offer); // mime->name so that we resolve aliases
+                        //qCDebug(SYCOCA) << "Adding service" << service->entryPath() << "to" << mime.name();
+                        m_offerHash.addServiceOffer(mime.name(), offer); // mime.name() so that we resolve aliases
                     }
                 }
             }
@@ -322,7 +320,7 @@ void KBuildServiceFactory::populateServiceTypes()
     KMimeAssociations mimeAssociations(m_offerHash, this);
     mimeAssociations.parseAllMimeAppsList();
 
-    // Now for each mimetype, collect services from parent mimetypes
+    // Now for each MIME type, collect services from parent MIME types
     collectInheritedServices();
 
     // Now collect the offsets into the (future) offer list
@@ -374,7 +372,7 @@ void KBuildServiceFactory::saveOfferList(QDataStream &str)
         const QString stName = it.key();
         const ServiceTypeOffersData offersData = it.value();
         QList<KServiceOffer> offers = offersData.offers;
-        qStableSort(offers);   // by initial preference
+        std::stable_sort(offers.begin(), offers.end());   // by initial preference
 
         int offset = -1;
         KServiceType::Ptr serviceType = m_serviceTypeFactory->findServiceTypeByName(stName);
@@ -388,14 +386,14 @@ void KBuildServiceFactory::saveOfferList(QDataStream &str)
             }
         }
         if (offset == -1) {
-            qCDebug(SYCOCA) << "Didn't find servicetype or mimetype" << stName;
+            qCDebug(SYCOCA) << "Didn't find servicetype or MIME type" << stName;
             continue;
         }
 
         for (QList<KServiceOffer>::const_iterator it2 = offers.constBegin();
                 it2 != offers.constEnd(); ++it2) {
 
-            //qCDebug(SYCOCA) << stName << ":" << "writing offer" << (*it2).service()->desktopEntryName() << offset << (*it2).service()->offset() << "in sycoca at pos" << str.device()->pos();
+            //qCDebug(SYCOCA) << stName << ": writing offer" << (*it2).service()->desktopEntryName() << offset << (*it2).service()->offset() << "in sycoca at pos" << str.device()->pos();
             Q_ASSERT((*it2).service()->offset() != 0);
 
             str << qint32(offset);
